package com.sf.service.goods.service.impl;

import java.util.List;
import com.sf.common.utils.DateUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.sf.service.goods.mapper.GoodsMessagesMapper;
import com.sf.service.goods.domain.GoodsMessages;
import com.sf.service.goods.service.IGoodsMessagesService;
import org.springframework.util.StringUtils;

/**
 * 商品信息Service业务层处理
 * 
 * @author zoukun
 * @date 2024-04-18
 */
@Service
public class GoodsMessagesServiceImpl implements IGoodsMessagesService 
{
    @Autowired
    private GoodsMessagesMapper goodsMessagesMapper;

    /**
     * 查询商品信息
     * 
     * @param id 商品信息主键
     * @return 商品信息
     */
    @Override
    public GoodsMessages selectGoodsMessagesById(Long id)
    {
        return goodsMessagesMapper.selectGoodsMessagesById(id);
    }

    /**
     * 查询商品信息列表
     * 
     * @param goodsMessages 商品信息
     * @return 商品信息
     */
    @Override
    public List<GoodsMessages> selectGoodsMessagesList(GoodsMessages goodsMessages)
    {
        return goodsMessagesMapper.selectGoodsMessagesList(goodsMessages);
    }

    /**
     * 新增商品信息
     * 
     * @param goodsMessages 商品信息
     * @return 结果
     */
    @Override
    public int insertGoodsMessages(GoodsMessages goodsMessages)
    {
        goodsMessages.setCreateTime(DateUtils.getNowDate());
        return goodsMessagesMapper.insertGoodsMessages(goodsMessages);
    }

    /**
     * 修改商品信息
     * 
     * @param goodsMessages 商品信息
     * @return 结果
     */
    @Override
    public int updateGoodsMessages(GoodsMessages goodsMessages)
    {
        goodsMessages.setUpdateTime(DateUtils.getNowDate());
        return goodsMessagesMapper.updateGoodsMessages(goodsMessages);
    }

    /**
     * 批量删除商品信息
     * 
     * @param ids 需要删除的商品信息主键
     * @return 结果
     */
    @Override
    public int deleteGoodsMessagesByIds(Long[] ids)
    {
        return goodsMessagesMapper.deleteGoodsMessagesByIds(ids);
    }

    /**
     * 删除商品信息信息
     * 
     * @param id 商品信息主键
     * @return 结果
     */
    @Override
    public int deleteGoodsMessagesById(Long id)
    {
        return goodsMessagesMapper.deleteGoodsMessagesById(id);
    }

    @Override
    public GoodsMessages selectGoodsMessagesByCode(String goodsCode) {
        if (StringUtils.hasText(goodsCode)){
            return goodsMessagesMapper.selectGoodsMessagesByCode(goodsCode);
        }
        return null;
    }
}
