package com.sf.vertx.handle;

import static com.sf.vertx.constans.SACConstants.CACHE_KEY_CONNECTOR;

import com.sf.vertx.api.pojo.ApiConfig;
import com.sf.vertx.api.pojo.AppConfig;
import com.sf.vertx.enums.GatewayError;
import com.sf.vertx.exception.ServiceException;

import io.vertx.ext.web.RoutingContext;
import lombok.extern.slf4j.Slf4j;

@Slf4j
public class OpenParameterCheckHandlerImpl extends BaseRoutingContextDataHandler implements ParameterCheckHandler {

	@Override
	public void handle(RoutingContext rc) {
		try {
			log.info("Enter OPEN Route");
			// 判断OPEN模式, header不传递参数, 走域名映射
			String domain = rc.request().authority().host();
			log.info("request domain:{}", domain);
			AppConfig appConfig = AppConfigHandler.getAppConfigByDomain(domain);
			if (appConfig == null) {
				rc.fail(new ServiceException(GatewayError.APP_SERVICE_NOT_FOUND));
				return;
			}
			String apiCacheKey = appConfig.getAppCode() + CACHE_KEY_CONNECTOR + domain;
			ApiConfig apiConfig = AppConfigHandler.getApicodeConfig(apiCacheKey);
			if (apiConfig == null) {
				rc.fail(new ServiceException(GatewayError.API_SERVICE_NOT_FOUND));
				return;
			}
			
			// 设置上下文数据
			this.ctxData(rc, appConfig, apiConfig, apiCacheKey);
			
			// 反向代理负载均衡、前置拦截器(uri rewrite),需要从header拿到appCode,apiCode
			rc.request().headers().add(AppConfigHandler.getAppCodeHeaderKey(), appConfig.getAppCode());
			rc.request().headers().add(AppConfigHandler.getApiCodeHeaderKey(), domain);
		} catch (Exception e) {
			log.error("OpenParameterCheckHandlerImpl Error:", e);
			rc.fail(new ServiceException(GatewayError.DEFAULT_SERVICE_ERROR));
			return;
		}
		rc.next();
	}
}
